@@ -21,13 +21,15 @@
 
 #include <pulsar/Reader.h>
 
-#include <boost/variant.hpp>
+#include <atomic>
 #include <cstdint>
 #include <functional>
 #include <list>
 #include <memory>
+#include <optional>
 #include <set>
 #include <utility>
+#include <variant>
 
 #include "BrokerConsumerStatsImpl.h"
 #include "Commands.h"
@@ -37,7 +39,6 @@
 #include "MapCache.h"
 #include "MessageIdImpl.h"
 #include "NegativeAcksTracker.h"
-#include "Synchronized.h"
 #include "TestUtil.h"
 #include "TimeUtils.h"
 #include "UnboundedBlockingQueue.h"
@@ -79,9 +80,9 @@ const static std::string DLQ_GROUP_TOPIC_SUFFIX = "-DLQ";
 
 enum class SeekStatus : std::uint8_t
 {
-    NOT_STARTED,
-    IN_PROGRESS,
-    COMPLETED
+    NOT_STARTED,  // there is no pending seek RPC so that it's allowed to seek
+    IN_PROGRESS,  // the seek RPC is in progress
+    COMPLETED     // the seek RPC is done but the connection is not established yet
 };
 
 class ConsumerImpl : public ConsumerImplBase {
@@ -138,7 +139,9 @@ class ConsumerImpl : public ConsumerImplBase {
     void getBrokerConsumerStatsAsync(const BrokerConsumerStatsCallback& callback) override;
     void getLastMessageIdAsync(const BrokerGetLastMessageIdCallback& callback) override;
     void seekAsync(const MessageId& msgId, const ResultCallback& callback) override;
-    void seekAsync(uint64_t timestamp, const ResultCallback& callback) override;
+    using SeekTimestampType = uint64_t;
+    using SeekArg = std::variant<SeekTimestampType, MessageId>;
+    void seekAsync(SeekTimestampType timestamp, const ResultCallback& callback) override;
     void negativeAcknowledge(const MessageId& msgId) override;
     bool isConnected() const override;
     uint64_t getNumberOfConnectedConsumer() override;
@@ -191,8 +194,10 @@ class ConsumerImpl : public ConsumerImplBase {
     void drainIncomingMessageQueue(size_t count);
     uint32_t receiveIndividualMessagesFromBatch(const ClientConnectionPtr& cnx, Message& batchedMessage,
                                                 const BitSet& ackSet, int redeliveryCount);
-    bool isPriorBatchIndex(int32_t idx);
-    bool isPriorEntryIndex(int64_t idx);
+    template <typename T>
+    bool isPrior(T index, T startIndex) const noexcept {
+        return config_.isStartMessageIdInclusive() ? (index < startIndex) : (index <= startIndex);
+    }
     void brokerConsumerStatsListener(Result, BrokerConsumerStatsImpl, const BrokerConsumerStatsCallback&);
 
     enum class DecryptionResult : uint8_t
@@ -218,19 +223,9 @@ class ConsumerImpl : public ConsumerImplBase {
                                        const BrokerGetLastMessageIdCallback& callback);
 
     void clearReceiveQueue();
-    using SeekArg = boost::variant<uint64_t, MessageId>;
-    friend std::ostream& operator<<(std::ostream& os, const SeekArg& seekArg) {
-        auto ptr = boost::get<uint64_t>(&seekArg);
-        if (ptr) {
-            os << *ptr;
-        } else {
-            os << *boost::get<MessageId>(&seekArg);
-        }
-        return os;
-    }
 
     void seekAsyncInternal(long requestId, const SharedBuffer& seek, const SeekArg& seekArg,
-                           const ResultCallback& callback);
+                           ResultCallback&& callback);
     void processPossibleToDLQ(const MessageId& messageId, const ProcessDLQCallBack& cb);
 
     std::mutex mutexForReceiveWithZeroQueueSize;
@@ -269,19 +264,13 @@ class ConsumerImpl : public ConsumerImplBase {
     std::shared_ptr<Promise<Result, Producer>> deadLetterProducer_;
     std::mutex createProducerLock_;
 
-    // Make the access to `lastDequedMessageId_` and `lastMessageIdInBroker_` thread safe
-    mutable std::mutex mutexForMessageId_;
     MessageId lastDequedMessageId_{MessageId::earliest()};
     MessageId lastMessageIdInBroker_{MessageId::earliest()};
+    optional<MessageId> startMessageId_;
 
-    std::atomic<SeekStatus> seekStatus_{SeekStatus::NOT_STARTED};
-    Synchronized<ResultCallback> seekCallback_{[](Result) {}};
-    Synchronized<optional<MessageId>> startMessageId_;
-    Synchronized<MessageId> seekMessageId_{MessageId::earliest()};
-    std::atomic<bool> hasSoughtByTimestamp_{false};
-
-    bool hasSoughtByTimestamp() const { return hasSoughtByTimestamp_.load(std::memory_order_acquire); }
-    bool duringSeek() const { return seekStatus_ != SeekStatus::NOT_STARTED; }
+    SeekStatus seekStatus_{SeekStatus::NOT_STARTED};
+    optional<ResultCallback> seekCallback_;
+    optional<SeekArg> lastSeekArg_;
 
     class ChunkedMessageCtx {
        public:
@@ -380,7 +369,8 @@ class ConsumerImpl : public ConsumerImplBase {
                                                const ClientConnectionPtr& cnx, MessageId& messageId);
 
     bool hasMoreMessages() const {
-        std::lock_guard<std::mutex> lock{mutexForMessageId_};
+        LockGuard lock{mutex_};
+
         if (lastMessageIdInBroker_.entryId() == -1L) {
             return false;
         }
@@ -388,7 +378,7 @@ class ConsumerImpl : public ConsumerImplBase {
         const auto inclusive = config_.isStartMessageIdInclusive();
         if (lastDequedMessageId_ == MessageId::earliest()) {
             // If startMessageId_ is none, use latest so that this method will return false
-            const auto startMessageId = startMessageId_.get().value_or(MessageId::latest());
+            const auto startMessageId = startMessageId_.value_or(MessageId::latest());
             return inclusive ? (lastMessageIdInBroker_ >= startMessageId)
                              : (lastMessageIdInBroker_ > startMessageId);
         } else {
@@ -396,11 +386,22 @@ class ConsumerImpl : public ConsumerImplBase {
         }
     }
 
+    auto getStartMessageId() const {
+        LockGuard lock{mutex_};
+        return startMessageId_;
+    }
+    auto setLastDequedMessageId(const MessageId& messageId) {
+        LockGuard lock{mutex_};
+        lastDequedMessageId_ = messageId;
+    }
+
     void doImmediateAck(const ClientConnectionPtr& cnx, const MessageId& msgId, CommandAck_AckType ackType,
                         const ResultCallback& callback);
     void doImmediateAck(const ClientConnectionPtr& cnx, const std::set<MessageId>& msgIds,
                         const ResultCallback& callback);
 
+    using LockGuard = std::lock_guard<std::mutex>;
+
     friend class PulsarFriend;
     friend class MultiTopicsConsumerImpl;
 
